@@ -20,13 +20,17 @@ class CpptrajRms(BiobbObject):
         input_traj_path (str): Path to the input trajectory to be processed.  File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/ambertools/cpptraj.traj.dcd>`_. Accepted formats: mdcrd (edam:format_3878), crd (edam:format_3878), cdf (edam:format_3650), netcdf (edam:format_3650), nc (edam:format_3650), restart (edam:format_3886), ncrestart (edam:format_3886), restartnc (edam:format_3886), dcd (edam:format_3878), charmm (edam:format_3887), cor (edam:format_2033), pdb (edam:format_1476), mol2 (edam:format_3816), trr (edam:format_3910), gro (edam:format_2033), binpos (edam:format_3885), xtc (edam:format_3875), cif (edam:format_1477), arc (edam:format_2333), sqm (edam:format_2033), sdf (edam:format_3814), conflib (edam:format_2033).
         input_exp_path (str) (Optional): Path to the experimental reference file (required if reference = experimental). File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/ambertools/experimental.1e5t.pdb>`_. Accepted formats: pdb (edam:format_1476).
         output_cpptraj_path (str): Path to the output processed analysis. File type: output. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/reference/ambertools/ref_cpptraj.rms.first.dat>`_. Accepted formats: dat (edam:format_1637), agr (edam:format_2033), xmgr (edam:format_2033), gnu (edam:format_2033).
+        output_traj_path (str) (Optional): Path to the output processed trajectory. File type: output. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/ambertools/cpptraj.traj.dcd>`_. Accepted formats: mdcrd (edam:format_3878), crd (edam:format_3878), cdf (edam:format_3650), netcdf (edam:format_3650), nc (edam:format_3650), restart (edam:format_3886), ncrestart (edam:format_3886), restartnc (edam:format_3886), dcd (edam:format_3878), charmm (edam:format_3887), cor (edam:format_2033), pdb (edam:format_1476), mol2 (edam:format_3816), trr (edam:format_3910), gro (edam:format_2033), binpos (edam:format_3885), xtc (edam:format_3875), cif (edam:format_1477), arc (edam:format_2333), sqm (edam:format_2033), sdf (edam:format_3814), conflib (edam:format_2033).
         properties (dic - Python dictionary object containing the tool parameters, not input/output files):
             * **start** (*int*) - (1) [1~100000|1] Starting frame for slicing
             * **end** (*int*) - (-1) [-1~100000|1] Ending frame for slicing
             * **steps** (*int*) - (1) [1~100000|1] Step for slicing
-            * **mask** (*str*) - ("all-atoms") Mask definition. Values: c-alpha (All c-alpha atoms; protein only), backbone (Backbone atoms), all-atoms (All system atoms), heavy-atoms (System heavy atoms; not hydrogen), side-chain (All not backbone atoms), solute (All system atoms except solvent atoms), ions (All ion molecules), solvent (All solvent atoms).
+            * **mask** (*str*) - ("all-atoms") Mask definition. Values: c-alpha (All c-alpha atoms; protein only), backbone (Backbone atoms), all-atoms (All system atoms), heavy-atoms (System heavy atoms; not hydrogen), side-chain (All not backbone atoms), solute (All system atoms except solvent atoms), ions (All ion molecules), solvent (All solvent atoms), AnyAmberFromatMask (Amber atom selection syntax like `@*`).
             * **reference** (*str*) - ("first") Reference definition. Values: first (Use the first trajectory frame as reference), average (Use the average of all trajectory frames as reference), experimental (Use the experimental structure as reference).
-            * **cpptraj_path** (*str*) - ("cpptraj") Path to the cpptraj executable binary.
+            * **nofit** (*bool*) - (False) Do not perform best-fit RMSD
+            * **norotate** (*bool*) - (False) Translate but do not rotate coordinates
+            * **nomod** (*bool*) - (False) Do not modify coordinates
+            * **binary_path** (*str*) - ("cpptraj") Path to the cpptraj executable binary.
             * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
             * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
             * **container_path** (*str*) - (None) Container path definition.
@@ -51,6 +55,7 @@ class CpptrajRms(BiobbObject):
                         input_traj_path='/path/to/myTrajectory.dcd', 
                         output_cpptraj_path='/path/to/newAnalysis.dat', 
                         input_exp_path= '/path/to/myExpStructure.pdb',
+                        output_traj_path='/path/to/newTrajectory.netcdf', 
                         properties=prop)
 
     Info:
@@ -65,16 +70,17 @@ class CpptrajRms(BiobbObject):
     """
 
     def __init__(self, input_top_path, input_traj_path, output_cpptraj_path, 
-                input_exp_path = None, properties=None, **kwargs) -> None:
+                input_exp_path = None, output_traj_path = None, properties=None, **kwargs) -> None:
         properties = properties or {}
 
         # Call parent class constructor
         super().__init__(properties)
+        self.locals_var_dict = locals().copy()
 
         # Input/Output files
         self.io_dict = { 
             "in": { "input_top_path": input_top_path, "input_traj_path": input_traj_path, "input_exp_path": input_exp_path }, 
-            "out": { "output_cpptraj_path": output_cpptraj_path } 
+            "out": { "output_cpptraj_path": output_cpptraj_path, "output_traj_path": output_traj_path } 
         }
 
         # Properties specific for BB
@@ -84,17 +90,23 @@ class CpptrajRms(BiobbObject):
         self.steps =  properties.get('steps', 1)
         self.mask = properties.get('mask', 'all-atoms')
         self.reference = properties.get('reference', 'first')
+        self.nofit = properties.get('nofit', False)
+        self.norotate = properties.get('norotate', False)
+        self.nomod = properties.get('nomod', False)
         self.properties = properties
-        self.cpptraj_path = get_binary_path(properties, 'cpptraj_path')
+        self.binary_path = get_binary_path(properties, 'binary_path')
 
         # Check the properties
         self.check_properties(properties)
+        self.check_arguments()
 
     def check_data_params(self, out_log, err_log):
         """ Checks all the input/output paths and parameters """
         self.io_dict["in"]["input_top_path"], self.input_top_path_orig = check_top_path(self.io_dict["in"]["input_top_path"], out_log, self.__class__.__name__)
         self.io_dict["in"]["input_traj_path"] = check_traj_path(self.io_dict["in"]["input_traj_path"], out_log, self.__class__.__name__)
         self.io_dict["out"]["output_cpptraj_path"] = check_out_path(self.io_dict["out"]["output_cpptraj_path"], out_log, self.__class__.__name__)
+        if self.io_dict["out"]["output_traj_path"]:
+            self.io_dict["out"]["output_traj_path"] = check_out_path(self.io_dict["out"]["output_traj_path"], out_log, self.__class__.__name__)
         self.in_parameters = { 'start': self.start, 'end': self.end, 'step': self.steps, 'mask': self.mask, 'reference': self.reference }
 
     def create_instructions_file(self, container_io_dict, out_log, err_log):
@@ -130,7 +142,12 @@ class CpptrajRms(BiobbObject):
         inp_exp_pth = None
         if "input_exp_path" in container_io_dict["in"]:
             inp_exp_pth = container_io_dict["in"]["input_exp_path"]
-        instructions_list += get_reference(reference, container_io_dict["out"]["output_cpptraj_path"], inp_exp_pth, ref_mask, True, self.__class__.__name__, out_log)
+        instructions_list += get_reference_rms(reference, container_io_dict["out"]["output_cpptraj_path"], inp_exp_pth, ref_mask, True,
+                                               self.__class__.__name__, out_log,  self.nofit, self.norotate, self.nomod)            
+
+        # trajout
+        if ("output_traj_path" in container_io_dict["out"]):
+            instructions_list.append('trajout ' + container_io_dict["out"]["output_traj_path"])
 
         # create .in file
         with open(self.instructions_file, 'w') as mdp:
@@ -158,7 +175,7 @@ class CpptrajRms(BiobbObject):
             copy_instructions_file_to_container(self.instructions_file, self.stage_io_dict['unique_dir'])
 
         # create cmd and launch execution
-        self.cmd = [self.cpptraj_path, '-i', self.instructions_file]
+        self.cmd = [self.binary_path, '-i', self.instructions_file]
 
         # Run Biobb block
         self.run_biobb()
@@ -167,13 +184,17 @@ class CpptrajRms(BiobbObject):
         self.copy_to_host()
 
         # remove temporary folder(s)
-        self.tmp_files = [PurePath(self.instructions_file).parent]
-        if self.container_path: self.tmp_files.append(self.stage_io_dict['unique_dir'])
-        remove_tmp_files(self.tmp_files, self.remove_tmp, self.out_log, self.input_top_path_orig, self.io_dict["in"]["input_top_path"])
+        self.tmp_files.extend([
+            self.stage_io_dict.get("unique_dir"),
+            PurePath(self.instructions_file).parent
+        ])
+        self.remove_tmp_files()
+
+        self.check_arguments(output_files_created=True, raise_exception=False)
 
         return self.return_code
 
-def cpptraj_rms(input_top_path: str, input_traj_path: str, output_cpptraj_path: str, input_exp_path: str = None, properties: dict = None, **kwargs) -> int:
+def cpptraj_rms(input_top_path: str, input_traj_path: str, output_cpptraj_path: str, input_exp_path: str = None, output_traj_path: str = None, properties: dict = None, **kwargs) -> int:
     """Execute the :class:`CpptrajRms <ambertools.cpptraj_rms.CpptrajRms>` class and
     execute the :meth:`launch() <ambertools.cpptraj_rms.CpptrajRms.launch>` method."""
 
@@ -181,6 +202,7 @@ def cpptraj_rms(input_top_path: str, input_traj_path: str, output_cpptraj_path:
                     input_traj_path=input_traj_path, 
                     output_cpptraj_path=output_cpptraj_path,
                     input_exp_path=input_exp_path,
+                    output_traj_path=output_traj_path,
                     properties=properties, **kwargs).launch()
 
 def main():
@@ -194,6 +216,7 @@ def main():
     required_args.add_argument('--input_traj_path', required=True, help='Path to the input trajectory to be processed. Accepted formats: crd, cdf, netcdf, restart, ncrestart, restartnc, dcd, charmm, cor, pdb, mol2, trr, gro, binpos, xtc, cif, arc, sqm, sdf, conflib.')
     parser.add_argument('--input_exp_path', required=False, help='Path to the experimental reference file (required if reference = experimental).')
     required_args.add_argument('--output_cpptraj_path', required=True, help='Path to the output processed analysis.')
+    parser.add_argument('--output_traj_path', required=False, help='Path to the output processed trajectory.')
 
     args = parser.parse_args()
     args.config = args.config or "{}"
@@ -204,6 +227,7 @@ def main():
                 input_traj_path=args.input_traj_path, 
                 output_cpptraj_path=args.output_cpptraj_path, 
                 input_exp_path=args.input_exp_path, 
+                output_traj_path=args.output_traj_path, 
                 properties=properties)
 
 if __name__ == '__main__':
